use std::collections::HashMap;
use std::fs::File;
use std::io::{BufReader, Read};
use std::path::Path;

pub type Symbols = HashMap<String, u32>;

impl Symbols {
    pub fn load(filename: Path) -> anyhow::Result<Symbols> {
        let file = File::open(filename)?;
        let mut reader = BufReader::new(file);

        reader.lines().filter_map(|l| l.ok().and_then(|line| {
            let words: Vec<String> = line.split_ascii_whitespace().collect();
            if words.len() == 2 {
                Some((words[1], words[0]))
            } else {
                None
            }
        }))
        .filter_map(|(symbol, mut address)| {
            if let Some(colon_at) = address.find(':') {
                address.remove(colon_at);
            }

            let snes_address = u32::from_str_radix(&address, 16).ok()?;

            let pc_address = (snes_address & 0x7FFF) + ((addr / 2) & 0xFF8000);

            Some((symbol, pc_address))
        }).collect()
    }
}
